@@ -26,6 +26,7 @@ from docassemble.base.util import (
     DAStaticFile,
     alpha,
 )
+from docassemble.base.pdfa import pdf_to_pdfa
 
 __all__ = [
     "ALAddendumField",
@@ -48,6 +49,10 @@ __all__ = [
 DEBUG_MODE = get_config("debug")
 
 
+def base_name(filename: str) -> str:
+    return os.path.splitext(filename)[0]
+
+
 def label(dictionary):
     """Given a dictionary like: {"some_attribute":"Some label"}, return the `value` of the first dictionary item.
     Useful for working with the `columns` method of an ALAddendumField.
@@ -651,17 +656,34 @@ class ALDocument(DADict):
             self.has_addendum = False
         self.initializeAttribute("cache", DALazyAttribute)
         self.always_enabled = hasattr(self, "enabled") and self.enabled
+        if not hasattr(self, "suffix_to_append"):
+            # When the key is "preview", append it to the file name
+            self.suffix_to_append = "preview"
 
-    def as_pdf(self, key: str = "final", refresh: bool = True) -> DAFile:
+    def as_pdf(
+        self,
+        key: str = "final",
+        refresh: bool = True,
+        pdfa: bool = False,
+        append_matching_suffix: bool = True,
+    ) -> DAFile:
         # Trigger some stuff up front to avoid idempotency problems
-        filename = self.filename
         self.title
         self.need_addendum()
-        if not filename.endswith(".pdf"):
-            filename += ".pdf"
 
         safe_key = space_to_underscore(key)
 
+        if append_matching_suffix and key == self.suffix_to_append:
+            append_suffix: str = f"_{safe_key}"
+        else:
+            append_suffix = ""
+        filename = f"{base_name(self.filename)}{append_suffix}.pdf"
+
+        # The PDF/A version of the document is a different file than the normal PDF,
+        # so differentiate that when checking the cache
+        if pdfa:
+            safe_key = safe_key + "-pdfa"
+
         if hasattr(self.cache, safe_key):
             return getattr(self.cache, safe_key)
 
@@ -673,7 +695,12 @@ class ALDocument(DADict):
         if isinstance(main_doc, DAFileCollection):
             main_doc = main_doc.pdf
             main_doc.title = self.title
-            main_doc.filename = filename  # Not sure if this works?
+            main_doc.filename = filename
+            try:
+                main_doc.set_attributes(filename=filename)
+                main_doc.set_mimetype("application/pdf")
+            except:
+                pass
 
         if self.need_addendum():
             if refresh:
@@ -682,21 +709,37 @@ class ALDocument(DADict):
                 addendum_doc = self.addendum
             if isinstance(main_doc, DAFileCollection):
                 addendum_doc = addendum_doc.pdf
-            concatenated = pdf_concatenate(main_doc, addendum_doc, filename=filename)
+            concatenated = pdf_concatenate(
+                main_doc, addendum_doc, filename=filename, pdfa=pdfa
+            )
             concatenated.title = self.title
             setattr(self.cache, safe_key, concatenated)
             return concatenated
         else:
+            if pdfa:
+                pdf_to_pdfa(main_doc.path())
             setattr(self.cache, safe_key, main_doc)
             return main_doc
 
-    def as_docx(self, key: str = "final", refresh: bool = True) -> DAFile:
+    def as_docx(
+        self,
+        key: str = "final",
+        refresh: bool = True,
+        append_matching_suffix: bool = True,
+    ) -> DAFile:
         """
         Returns the assembled document as a single DOCX file, if possible. Otherwise returns a PDF.
         """
+        if append_matching_suffix and key == self.suffix_to_append:
+            filename = f"{base_name(self.filename)}_{key}"
+        else:
+            filename = f"{base_name(self.filename)}"
         if self.need_addendum():
             try:
-                the_file = docx_concatenate(self.as_list(key=key, refresh=refresh))
+                the_file = docx_concatenate(
+                    self.as_list(key=key, refresh=refresh),
+                    filename=filename + ".docx",
+                )
                 the_file.title = self.title
                 return the_file
             except:
@@ -705,9 +748,10 @@ class ALDocument(DADict):
         if self._is_docx(key=key):
             the_file = self[key].docx
             the_file.title = self.title
+            the_file.set_attributes(filename=filename + ".docx")
             return the_file
 
-        return self.as_pdf(key=key)
+        return self.as_pdf(key=key, append_matching_suffix=append_matching_suffix)
 
     def _is_docx(self, key: str = "final"):
         """Returns True iff the file is a DOCX."""
@@ -786,6 +830,25 @@ class ALDocument(DADict):
             input_width=input_width,
         )
 
+    def is_enabled(self, refresh=True):
+        """
+        A document can be considered "enabled" if:
+        - the .always_enabled attribute is true (enabled at init)
+        - the .enabled attribute is true (calculated fresh once per page load)
+        - the cache.enabled attribute is true
+        """
+        if hasattr(self, "always_enabled") and self.always_enabled:
+            return True
+        if hasattr(self.cache, "enabled"):
+            return self.cache.enabled
+        if refresh:
+            self.cache.enabled = self.enabled
+            if hasattr(self, "enabled"):
+                del self.enabled
+            return self.cache.enabled
+        else:
+            return self.enabled
+
 
 class ALStaticDocument(DAStaticFile):
     """A class that allows one-line initialization of static documents to include in an ALDocumentBundle.
@@ -820,6 +883,9 @@ class ALStaticDocument(DAStaticFile):
         self.gathered = True
         self.initializeAttribute("cache", DALazyAttribute)
         self.always_enabled = hasattr(self, "enabled") and self.enabled
+        if not hasattr(self, "suffix_to_append"):
+            # When the key is "preview", append it to the file name
+            self.suffix_to_append = "preview"
 
     def __getitem__(self, key):
         # This overrides the .get() method so that the 'final' and 'private' key always exist and
@@ -829,19 +895,36 @@ class ALStaticDocument(DAStaticFile):
     def as_list(self, key: str = "final", refresh: bool = True) -> List[DAStaticFile]:
         return [self]
 
-    def as_pdf(self, key: str = "final", refresh: bool = True) -> DAStaticFile:
-        return pdf_concatenate(self)
+    def as_pdf(
+        self,
+        key: str = "final",
+        pdfa: bool = False,
+        filename: str = "",
+        append_matching_suffix: bool = True,
+        refresh: bool = False,
+    ) -> Union[DAStaticFile, DAFile]:
+        if not filename:
+            filename = self.filename
+        return pdf_concatenate(self, pdfa=pdfa, filename=f"{base_name(filename)}.pdf")
 
     def as_docx(
-        self, key: str = "final", refresh: bool = True
+        self,
+        key: str = "final",
+        refresh: bool = True,
+        append_matching_suffix: bool = False,
     ) -> Union[DAStaticFile, DAFile]:
         """
         Returns the assembled document as a single DOCX file, if possible. Otherwise returns a PDF.
+        The "append_matching_suffix" parameter is not used for static documents. They are always
+        left unchanged.
         """
         if self._is_docx():
             return self
         else:
-            return self.as_pdf(key=key)
+            # Don't apply the append_matching_suffix strategy to static files because it would
+            # be difficult to make consistent between DOCX and PDF, and would have
+            # negative performance implications. By definition static files have only one version
+            return self.as_pdf(key=key, append_matching_suffix=False)
 
     def _is_docx(self, key: str = "final"):
         if hasattr(self, "extension") and self.extension.lower() == "docx":
@@ -859,6 +942,9 @@ class ALStaticDocument(DAStaticFile):
         # Workaround for problem generating thumbnails without it
         return pdf_concatenate(self).show(**kwargs)
 
+    def is_enabled(self, **kwargs) -> bool:
+        return self.enabled
+
 
 class ALDocumentBundle(DAList):
     """
@@ -893,30 +979,53 @@ class ALDocumentBundle(DAList):
         self.always_enabled = hasattr(self, "enabled") and self.enabled
         # Pre-cache some DALazyTemplates we set up to aid translation that won't
         # vary at runtime
+        if not hasattr(self, "suffix_to_append"):
+            # When the key is "preview", append it to the file name
+            self.suffix_to_append = "preview"
 
-    def as_pdf(self, key: str = "final", refresh: bool = True) -> Optional[DAFile]:
+    def as_pdf(
+        self,
+        key: str = "final",
+        refresh: bool = True,
+        pdfa: bool = False,
+        append_matching_suffix: bool = True,
+    ) -> Optional[DAFile]:
         """Returns the Bundle as a single PDF DAFile, or None if none of the documents are enabled."""
         safe_key = space_to_underscore(key)
+        if pdfa:
+            safe_key = safe_key + "-pdfa"
 
         if hasattr(self.cache, safe_key):
             return getattr(self.cache, safe_key)
 
-        if self.filename.endswith(".pdf"):
-            ending = ""
+        # When running automated tests, the "preview" version of the file is
+        # downloaded along with the final. Previously, the final version
+        # ovewrote the preview version. This makes the tests more useful by
+        # appending _preview to the name of the preview document.
+        if not hasattr(self, "suffix_to_append"):  # for existing interviews
+            self.suffix_to_append = "preview"
+        if append_matching_suffix and key == self.suffix_to_append:
+            append_suffix: str = f"_{key}"
         else:
-            ending = ".pdf"
+            append_suffix = ""
         files = self.enabled_documents(refresh=refresh)
         if len(files) == 0:
             # In the case of no enabled files, avoid errors
             return None
         elif len(files) == 1:
             # This case is simplest--we do not need to process the document at this level
-            pdf = files[0].as_pdf(key=key, refresh=refresh)
+            pdf = files[0].as_pdf(
+                key=key,
+                refresh=refresh,
+                pdfa=pdfa,
+                append_matching_suffix=append_matching_suffix,
+            )
             pdf.title = self.title
         else:
             pdf = pdf_concatenate(
                 [document.as_pdf(key=key, refresh=refresh) for document in files],
-                filename=self.filename + ending,
+                filename=f"{base_name(self.filename)}{append_suffix}.pdf",
+                pdfa=pdfa,
             )
         pdf.title = self.title
         setattr(self.cache, safe_key, pdf)
@@ -930,6 +1039,7 @@ class ALDocumentBundle(DAList):
         self,
         key: str = "final",
         refresh: bool = True,
+        pdfa: bool = False,
         title: str = "",
         format="pdf",
         include_pdf=True,
@@ -955,7 +1065,11 @@ class ALDocumentBundle(DAList):
             docs = [doc[key] for doc in self.enabled_documents(refresh=refresh)]
         else:
             docs = [
-                doc.as_pdf(key=key, refresh=refresh)
+                doc.as_pdf(
+                    key=key,
+                    refresh=refresh,
+                    pdfa=pdfa,
+                )
                 for doc in self.enabled_documents(refresh=refresh)
             ]
         zip = zip_file(docs, filename=zipname + ".zip")
@@ -970,6 +1084,14 @@ class ALDocumentBundle(DAList):
     def preview(self, refresh: bool = True) -> Optional[DAFile]:
         return self.as_pdf(key="preview", refresh=refresh)
 
+    def has_enabled_documents(self, refresh=False) -> bool:
+        """
+        Return True iff there is at least one enabled document
+        in this bundle.
+        """
+        # Use a generator expression for speed
+        return any(document.is_enabled(refresh=refresh) for document in self.elements)
+
     def enabled_documents(self, refresh: bool = True) -> List[Any]:
         """
         Returns the enabled documents
@@ -977,29 +1099,11 @@ class ALDocumentBundle(DAList):
         Args:
             refresh(bool): Controls whether the 'enabled' attribute is reconsidered.
         """
-        if refresh:
-            retval = []
-            for document in self.elements:
-                if document.always_enabled:
-                    enabled = True
-                else:
-                    if hasattr(document.cache, "enabled"):
-                        enabled = document.cache.enabled
-                    else:
-                        document.cache.enabled = document.enabled
-                        enabled = document.cache.enabled
-                    if hasattr(document, "enabled"):
-                        del document.enabled
-                if enabled:
-                    retval.append(document)
-            return retval
-        else:
-            return [
-                document
-                for document in self.elements
-                if (hasattr(document, "always_enabled") and document.always_enabled)
-                or document.enabled
-            ]
+        return [
+            document
+            for document in self.elements
+            if document.is_enabled(refresh=refresh)
+        ]
 
     def as_flat_list(self, key: str = "final", refresh: bool = True) -> List[DAFile]:
         """
@@ -1030,12 +1134,14 @@ class ALDocumentBundle(DAList):
                 flat_list.append(document.title)
         return flat_list
 
-    def as_pdf_list(self, key: str = "final", refresh: bool = True) -> List[DAFile]:
+    def as_pdf_list(
+        self, key: str = "final", refresh: bool = True, pdfa: bool = False
+    ) -> List[DAFile]:
         """
         Returns the nested bundles as a list of PDFs that is only one level deep.
         """
         return [
-            doc.as_pdf(key=key, refresh=refresh)
+            doc.as_pdf(key=key, refresh=refresh, pdfa=pdfa)
             for doc in self.enabled_documents(refresh=refresh)
         ]
 
@@ -1074,6 +1180,7 @@ class ALDocumentBundle(DAList):
         format: str = "pdf",
         view: bool = True,
         refresh: bool = True,
+        pdfa: bool = False,
         include_zip: bool = True,
         view_label="View",
         view_icon: str = "eye",
@@ -1081,6 +1188,7 @@ class ALDocumentBundle(DAList):
         download_icon: str = "download",
         zip_label: str = None,
         zip_icon: str = "file-archive",
+        append_matching_suffix: bool = True,
     ) -> str:
         """
         Returns string of a table to display a list
@@ -1100,7 +1208,10 @@ class ALDocumentBundle(DAList):
             doc.title
             if format == "pdf":
                 doc.as_pdf(
-                    key=key, refresh=refresh
+                    key=key,
+                    refresh=refresh,
+                    pdfa=pdfa,
+                    append_matching_suffix=append_matching_suffix,
                 )  # Generate cached file for this session
 
         html = f'<table class="al_table" id="{ html_safe_str(self.instanceName) }">'
@@ -1112,10 +1223,14 @@ class ALDocumentBundle(DAList):
                 download_doc = doc[key]
                 download_filename = doc.filename
             if format == "docx" and doc._is_docx(key=key):
-                download_doc = doc.as_docx(key=key)
+                download_doc = doc.as_docx(
+                    key=key, append_matching_suffix=append_matching_suffix
+                )
                 download_filename = filename_root + ".docx"
             else:
-                download_doc = doc.as_pdf(key=key)
+                download_doc = doc.as_pdf(
+                    key=key, append_matching_suffix=append_matching_suffix
+                )
                 download_filename = filename_root + ".pdf"
 
             try:
@@ -1146,7 +1261,9 @@ class ALDocumentBundle(DAList):
             if view and doc.as_pdf().url_for().endswith(".pdf"):
                 doc_view_button = action_button_html(
                     doc.as_pdf(key=key).url_for(
-                        attachment=False, display_filename=filename_root + ".pdf"
+                        attachment=False,
+                        display_filename=filename_root + ".pdf",
+                        append_matching_suffix=append_matching_suffix,
                     ),
                     label=view_label,
                     icon=view_icon,
@@ -1185,6 +1302,7 @@ class ALDocumentBundle(DAList):
         self,
         key: str = "final",
         format: str = "pdf",
+        pdfa: bool = False,
         view: bool = True,
         refresh: bool = True,
         view_label: str = "View",
@@ -1199,7 +1317,7 @@ class ALDocumentBundle(DAList):
         if format == "docx":
             the_file = self.as_docx(key=key)
         else:
-            the_file = self.as_pdf(key=key)
+            the_file = self.as_pdf(key=key, pdfa=pdfa)
 
         doc_download_button = action_button_html(
             the_file.url_for(attachment=True),
@@ -1244,6 +1362,8 @@ class ALDocumentBundle(DAList):
         Optionally, display a checkbox that allows someone to decide whether or not to
         include an editable (Word) copy of the file, iff it is available.
         """
+        if not self.has_enabled_documents():
+            return ""  # Don't let people email an empty set of documents
         if not hasattr(self, "_cached_get_email_copy"):
             self._cached_get_email_copy = str(self.get_email_copy)
         if not hasattr(self, "_cached_include_editable_documents"):
@@ -1325,11 +1445,30 @@ class ALDocumentBundle(DAList):
                 **kwargs,
             )
 
+    def _is_self_enabled(self, refresh=True) -> bool:
+        """The same as ALDocument.is_enabled"""
+        if hasattr(self, "always_enabled") and self.always_enabled:
+            return True
+        if hasattr(self.cache, "enabled"):
+            return self.cache.enabled
+        if refresh:
+            self.cache.enabled = self.enabled
+            if hasattr(self, "enabled"):
+                del self.enabled
+            return self.cache.enabled
+        else:
+            return self.enabled
+
+    def is_enabled(self, refresh=True) -> bool:
+        """Returns true if the bundle itself is enabled, and it has at least one enabled child document"""
+        self_enabled = self._is_self_enabled(refresh=refresh)
+        return self_enabled and self.has_enabled_documents(refresh=refresh)
+
 
 class ALExhibit(DAObject):
     """Class to represent a single exhibit, with cover page, which may contain multiple documents representing pages.
     Atributes:
-        elements (list): List of individual DAFiles representing uploaded images or documents.
+        pages (list): List of individual DAFiles representing uploaded images or documents.
         cover_page (DAFile | DAFileCollection): (optional) A DAFile or DAFileCollection object created by an `attachment:` block
           Will typically say something like "Exhibit 1"
         label (str): A label, like "A" or "1" for this exhibit in the cover page and table of contents
@@ -1342,13 +1481,42 @@ class ALExhibit(DAObject):
         self.object_type = DAFileList
         if not hasattr(self, "starting_page"):
             self.start_page = 1
+        if not hasattr(self, "suffix_to_append"):
+            # When the key is "preview", append it to the file name
+            self.suffix_to_append = "preview"
 
-    def add_numbers(self, starting_page=None) -> None:
+    def _start_ocr(self):
         """
-        Todo:
-            Not implemented yet.
+        Starts the OCR (optical character resolution) process on the uploaded documents.
+
+        Makes a background action for each page in the document.
         """
-        pass
+        if len(self.pages):
+            # We cannot OCR in place. It is too fragile.
+            for page in self.pages:
+                page.ocr_version = DAFile(page.attr_name("ocr_version"))
+                # psm=1 is the default which uses automatic text orientation and location detection.
+                # Appears to be the most accurate method.
+                page.ocr_status = page.ocr_version.make_ocr_pdf_in_background(
+                    page, psm=1
+                )
+
+    def ocr_ready(self) -> bool:
+        """
+        Returns:
+            True iff OCR process has finished on all pages. OCR is non-blocking, and assembly will work
+            even if OCR is not complete. Check this status if you want to wait to deliver a document until
+            OCR is complete.
+
+            Will return true (but log a warning) if OCR was never started on the documents.
+            That situation is likely a developer error, as you shouldn't wait for OCR if it never started
+        """
+        for page in self.pages:
+            if hasattr(page, "ocr_status") and not page.ocr_status.ready():
+                return False
+            if not hasattr(page, "ocr_status"):
+                log("developer warning: ocr_ready was called but _ocr_start wasn't!")
+        return True
 
     def ocr_pages(self):
         """
@@ -1370,22 +1538,47 @@ class ALExhibit(DAObject):
 
     def as_pdf(
         self,
-        prefix="",
+        *,
+        refresh: bool = False,
+        prefix: str = "",
+        pdfa: bool = False,
         add_page_numbers: bool = True,
         add_cover_page: bool = True,
         filename: str = None,
+        append_matching_suffix: bool = True,
     ) -> DAFile:
-        if hasattr(self._cache, "_file"):
-            return self._cache._file
+        """
+        Params:
+            prefix (str): the prefix for the bates numbering that is applied if
+              add_page_numbers is true
+            add_page_numbers (bool): adds bates numbering to the exhibit if true,
+              starting at the number self.start_page
+            add_cover_page (bool): adds a cover to this exhibit if true
+        """
+        safe_key = "_file"
+        if pdfa:
+            safe_key = safe_key + "_pdfa"
+        if add_page_numbers:
+            safe_key = safe_key + "_page_nums"
+
+        if hasattr(self._cache, safe_key):
+            return getattr(self._cache, safe_key)
         if not filename:
             filename = "exhibits.pdf"
         if add_cover_page:
-            self._cache._file = pdf_concatenate(
-                self.cover_page, self.ocr_pages(), filename=filename
+            concatenated_pages = pdf_concatenate(
+                self.cover_page, self.ocr_pages(), filename=filename, pdfa=pdfa
             )
         else:
-            self._cache._file = pdf_concatenate(self.ocr_pages(), filename=filename)
-        return self._cache._file
+            concatenated_pages = pdf_concatenate(
+                self.ocr_pages(), filename=filename, pdfa=pdfa
+            )
+
+        if add_page_numbers:
+            concatenated_pages.bates_number(prefix=prefix, start=self.start_page)
+
+        setattr(self._cache, safe_key, concatenated_pages)
+        return getattr(self._cache, safe_key)
 
     def num_pages(self) -> int:
         return self.pages.num_pages()
@@ -1418,11 +1611,27 @@ class ALExhibitList(DAList):
         if not hasattr(self, "auto_labeler"):
             self.auto_labeler = alpha
         if not hasattr(self, "auto_ocr"):
-            self.auto_ocr = True
+            self.auto_ocr = False
+        if not hasattr(self, "include_table_of_contents"):
+            self.include_table_of_contents = True
+        if not hasattr(self, "include_exhibit_cover_pages"):
+            self.include_exhibit_cover_pages = True
+        if not hasattr(self, "bates_prefix"):
+            self.bates_prefix = ""
         self.object_type = ALExhibit
         self.complete_attribute = "complete"
+        if not hasattr(self, "suffix_to_append"):
+            # When the key is "preview", append it to the file name
+            self.suffix_to_append = "preview"
 
-    def as_pdf(self, filename="file.pdf", add_cover_pages: bool = True) -> DAFile:
+    def as_pdf(
+        self,
+        filename="file.pdf",
+        pdfa: bool = False,
+        add_page_numbers: bool = False,
+        toc_pages: int = 0,
+        append_matching_suffix: bool = True,
+    ) -> DAFile:
         """
         Return a single PDF containing all exhibits.
         Args:
@@ -1431,21 +1640,21 @@ class ALExhibitList(DAList):
         Returns:
             A DAfile containing the rendered exhibit list as a single file.
         """
+        if self.include_table_of_contents and toc_pages != 1:
+            self._update_page_numbers(toc_guess_pages=toc_pages)
         return pdf_concatenate(
-            [exhibit.as_pdf(add_cover_page=add_cover_pages) for exhibit in self],
+            [
+                exhibit.as_pdf(
+                    add_cover_page=self.include_exhibit_cover_pages,
+                    add_page_numbers=add_page_numbers,
+                    prefix=self.bates_prefix,
+                )
+                for exhibit in self
+            ],
             filename=filename,
+            pdfa=pdfa,
         )
 
-    def add_numbers(self, prefix: str = "", starting_number: int = 1) -> None:
-        """
-        Add running page numbers. (TODO: not yet implemented)
-        Args:
-            prefix (str): The prefix before each page number. E.g., Ex-
-            starting_number (int): The number that the first page will be assigned. Defaults
-                                   to 1.
-        """
-        pass
-
     def _update_labels(self, auto_labeler: Callable = None) -> None:
         """
         Private method to refresh labels on all exhibits.
@@ -1460,38 +1669,30 @@ class ALExhibitList(DAList):
 
     def ocr_ready(self) -> bool:
         """
-        Returns:
-            True iff OCR process has finished on all pages. OCR is non-blocking, and assembly will work
-            even if OCR is not complete. Check this status if you want to wait to deliver a document until
-            OCR is complete.
+        Returns `True` iff all individual exhibit pages have been OCRed, or if the OCR process hasn't started.
         """
         ready = True
         for exhibit in self.elements:
-            for page in exhibit.pages:
-                if hasattr(page, "ocr_status"):
-                    ready &= page.ocr_status.ready()
+            ready &= exhibit.ocr_ready()
         return ready
 
-    def _update_page_numbers(self) -> None:
+    def _update_page_numbers(
+        self, starting_number: Optional[int] = None, toc_guess_pages: int = 1
+    ) -> None:
         """
         Update the `start_page` attribute of all exhibits so it reflects current position in the list + number of pages of each document.
         """
-        current_index = 1
+        toc_pages = toc_guess_pages if self.include_table_of_contents else 0
+        cover_pages = 1 if self.include_exhibit_cover_pages else 0
+        current_index = starting_number if starting_number else 1
+        current_index += toc_pages
         for exhibit in self.elements:
             exhibit.start_page = current_index
-            current_index += exhibit.num_pages()
+            current_index = current_index + exhibit.num_pages() + cover_pages
 
-    def _ocr_docs(self):
+    def _start_ocr(self):
         for exhibit in self.elements:
-            if len(exhibit.pages):
-                # We cannot OCR in place. It is too fragile.
-                for page in exhibit.pages:
-                    page.ocr_version = DAFile(page.attr_name("ocr_version"))
-                    # psm=1 is the default which uses automatic text orientation and location detection.
-                    # Appears to be the most accurate method.
-                    page.ocr_status = page.make_ocr_pdf_in_background(
-                        page.ocr_version, psm=1
-                    )
+            exhibit._start_ocr()
 
     def hook_after_gather(self):
         """
@@ -1501,11 +1702,8 @@ class ALExhibitList(DAList):
             self._update_page_numbers()
             if self.auto_label:
                 self._update_labels()
-
-        # TODO: implement below. Was buggy when OCRing a lot of files and OCRing in place.
-        # Switched to creating a new DAFile, but that introduced different bugs. -- third refresh breaks it. file never gets a number so not "ok"
-        # if self.auto_ocr:
-        #  self._ocr_docs()
+            if self.auto_ocr:
+                self._start_ocr()
 
 
 class ALExhibitDocument(ALDocument):
@@ -1554,11 +1752,26 @@ class ALExhibitDocument(ALDocument):
         self.initializeAttribute("exhibits", ALExhibitList)
         if hasattr(self, "auto_labeler"):
             self.exhibits.auto_labeler = self.auto_labeler
-        if not hasattr(self, "include_table_of_contents"):
-            self.include_table_of_contents = True
-        if not hasattr(self, "include_exhibit_cover_pages"):
+        if hasattr(self, "auto_ocr"):
+            self.exhibits.auto_ocr = self.auto_ocr
+        if hasattr(self, "bates_prefix"):
+            self.exhibits.bates_prefix = self.bates_prefix
+        if hasattr(self, "include_exhibit_cover_pages"):
+            self.exhibits.include_exhibit_cover_pages = self.include_exhibit_cover_pages
+        else:
             self.include_exhibit_cover_pages = True
+            self.exhibits.include_exhibit_cover_pages = True
+        if hasattr(self, "include_table_of_contents"):
+            self.exhibits.include_table_of_contents = self.include_table_of_contents
+        else:
+            self.include_table_of_contents = True
+            self.exhibits.include_table_of_contents = True
+        if not hasattr(self, "add_page_numbers"):
+            self.add_page_numbers = False
         self.has_addendum = False
+        if not hasattr(self, "suffix_to_append"):
+            # When the key is "preview", append it to the file name
+            self.suffix_to_append = "preview"
 
     def has_overflow(self):
         """
@@ -1566,6 +1779,12 @@ class ALExhibitDocument(ALDocument):
         """
         return False
 
+    def ocr_ready(self) -> bool:
+        """
+        Returns `True` iff each individual exhibit has been OCRed, or if the OCR process was not started.
+        """
+        return self.exhibits.ocr_ready()
+
     def __getitem__(self, key):
         # This overrides the .get() method so that the 'final' and 'private' key always exist and
         # point to the same file.
@@ -1574,29 +1793,49 @@ class ALExhibitDocument(ALDocument):
     def as_list(self, key: str = "final", refresh: bool = True) -> List[DAFile]:
         return [self]
 
-    def as_pdf(self, key="final", refresh: bool = True) -> DAFile:
+    def as_pdf(
+        self,
+        key="final",
+        refresh: bool = True,
+        pdfa: bool = False,
+        append_matching_suffix: bool = True,
+    ) -> DAFile:
         """
         Args:
             key (str): unused, for signature compatibility with ALDocument
+            refresh (bool): unused, for signature compatibility with ALDocument
         """
-        filename = os.path.splitext(self.filename)[0] + ".pdf"
+        if not hasattr(self, "suffix_to_append"):
+            self.suffix_to_append = "preview"
+        if append_matching_suffix and key == self.suffix_to_append:
+            filename = f"{base_name(self.filename)}_{key}.pdf"
+        else:
+            filename = base_name(self.filename) + ".pdf"
 
         if len(self.exhibits):
             if self.include_table_of_contents:
+                toc_pages = self.table_of_contents.num_pages()
                 return pdf_concatenate(
                     self.table_of_contents,
                     self.exhibits.as_pdf(
-                        add_cover_pages=self.include_exhibit_cover_pages
+                        add_page_numbers=self.add_page_numbers, toc_pages=toc_pages
                     ),
                     filename=filename,
+                    pdfa=pdfa,
                 )
             else:
                 return self.exhibits.as_pdf(
-                    add_cover_pages=self.include_exhibit_cover_pages, filename=filename
+                    add_page_numbers=self.add_page_numbers,
+                    filename=filename,
+                    pdfa=pdfa,
                 )
-        # pdf_concatenate([a.as_pdf() for a in self.exhibits], filename=self.filename)
 
-    def as_docx(self, key: str = "bool", refresh: bool = True) -> DAFile:
+    def as_docx(
+        self,
+        key: str = "bool",
+        refresh: bool = True,
+        append_matching_suffix: bool = True,
+    ) -> DAFile:
         return self.as_pdf()
 
 
@@ -1604,6 +1843,9 @@ class ALTableDocument(ALDocument):
     def init(self, *pargs, **kwargs):
         super().init(*pargs, **kwargs)
         self.has_addendum = False
+        if not hasattr(self, "suffix_to_append"):
+            # When the key is "preview", append it to the file name
+            self.suffix_to_append = "preview"
 
     def has_overflow(self):
         """
@@ -1621,19 +1863,34 @@ class ALTableDocument(ALDocument):
     ) -> List[DAFile]:
         return [self[key]]
 
-    def as_pdf(self, key: str = "final", refresh: bool = True, **kwargs) -> DAFile:
+    def as_pdf(
+        self,
+        key: str = "final",
+        refresh: bool = True,
+        pdfa: bool = False,
+        append_matching_suffix: bool = True,
+        **kwargs,
+    ) -> DAFile:
         """
         Args:
             key (str): unused, for signature compatibility with ALDocument
         """
+        if not hasattr(self, "suffix_to_append"):
+            # When the key is "preview", append it to the file name
+            self.suffix_to_append = "preview"
         if hasattr(self, "file"):
             return self.file
         self.file: DAFile = self.table.export(
-            self.filename + ".xlsx", title=self.filename
+            base_name(self.filename) + ".xlsx", title=self.filename
         )
         return self.file
 
-    def as_docx(self, key: str = "final", refresh: bool = True, **kwargs) -> DAFile:
+    def as_docx(
+        self,
+        key: str = "bool",
+        refresh: bool = True,
+        append_matching_suffix: bool = True,
+    ) -> DAFile:
         return self.as_pdf()
 
 
@@ -1641,6 +1898,8 @@ class ALUntransformedDocument(ALDocument):
     def init(self, *pargs, **kwargs):
         super().init(*pargs, **kwargs)
         self.has_addendum = False
+        if not hasattr(self, "suffix_to_append"):
+            self.suffix_to_append = "preview"
 
     def has_overflow(self):
         """
@@ -1653,14 +1912,26 @@ class ALUntransformedDocument(ALDocument):
     ) -> List[DAFile]:
         return [self[key]]
 
-    def as_pdf(self, key: str = "final", refresh: bool = True, **kwargs) -> DAFile:
+    def as_pdf(
+        self,
+        key: str = "final",
+        refresh: bool = True,
+        pdfa: bool = False,
+        append_matching_suffix: bool = True,
+        **kwargs,
+    ) -> DAFile:
         """
         Args:
             key (str): unused, for signature compatibility with ALDocument
         """
         return self[key]
 
-    def as_docx(self, key: str = "final", refresh: bool = True, **kwargs) -> DAFile:
+    def as_docx(
+        self,
+        key: str = "bool",
+        refresh: bool = True,
+        append_matching_suffix: bool = True,
+    ) -> DAFile:
         return self[key]
 
 
